#!/usr/bin/env python
'''
This sample demonstrates defect detection by image processing.

Usage:
  defect_detection.py

'''
import cv2 as cv
import numpy as np
import argparse

def main():
  img = cv.imread('/usr/share/opencv4/defect_detection/defect.png')
  img2 = cv.resize(img, (600,400))
  cv.imshow('0.Resize', img2)
  gray_mat = cv.cvtColor(img2, cv.COLOR_BGR2GRAY)
  # cv.imshow('1.Grey-scale Map', gray_mat)
  _retval, bin_mat = cv.threshold(gray_mat, 128, 255, cv.THRESH_BINARY)
  # cv.imshow('2.Binary Image', bin_mat)
  inv_mat = 255 - bin_mat
  # cv.imshow('3.Color Flip', inv_mat)
  element = cv.getStructuringElement(cv.MORPH_RECT, (3, 3))
  erode_mat = cv.erode(inv_mat, element)
  # cv.imshow("04.Erode", erode_mat);

  # Find the outline from erode_mat，and save the result to contours
  contours, hierarchy = cv.findContours(erode_mat, cv.RETR_TREE, cv.CHAIN_APPROX_SIMPLE)
  #drawing = np.zeros((erode_mat.shape[0], erode_mat.shape[1], 3), dtype=np.uint8)

  # Iterate over all contours and highline the defect contours.
  for i, c in enumerate(contours):
      aero = cv.contourArea(c)
      if  aero > 1000:
        cv.drawContours(img2, contours, i, (0, 0, 255), -1, 8, hierarchy)
        print('aero:', aero)

  cv.imshow("05.Contours Image", img2);
  cv.waitKey(0)
  print('Done')

if __name__ == '__main__':
    print(__doc__)
    main()
    cv.destroyAllWindows()
