#include <opencv2/dnn.hpp>
#include <opencv2/core.hpp>
#include <opencv2/videoio.hpp>
#include <opencv2/highgui.hpp>
#include <opencv2/imgproc.hpp>
#include <iostream>
#include <stdio.h>
#include "common.hpp"

// Namespaces.
using namespace cv;
using namespace std;
using namespace dnn;

// Text parameters.
const float FONT_SCALE = 0.7;
const int FONT_FACE = FONT_HERSHEY_SIMPLEX;
const int THICKNESS = 1;

// Colors.
Scalar BLACK = Scalar(0,0,0);
Scalar BLUE = Scalar(255, 178, 50);
Scalar YELLOW = Scalar(0, 255, 255);
Scalar RED = Scalar(0,0,255);

std::string keys =
    "{ help  h     | | Print help message. }"
    "{ device      |  4 | camera device number, 1 for CSI Camera with ISP, 4 for UVC in common case. }";

int main(int argc, char** argv)
{
    CommandLineParser parser(argc, argv, keys);
    parser.about("Use this script to run edge detection using OpenCV.");
    if (parser.has("help"))
    {
        parser.printMessage();
        return 0;
    }
       
    dnn::Net net;
    net = dnn::readNet("/usr/share/opencv4/mobileNetSSD/MobileNetSSD_deploy.caffemodel", "/usr/share/opencv4/mobileNetSSD/MobileNetSSD_deploy.prototxt");
    /* Object detection categories */    
    vector<String> objName;
    objName.push_back("background");
    objName.push_back("aeroplane");
    objName.push_back("bicycle");
    objName.push_back("bird");
    objName.push_back("boat");
    objName.push_back("bottle");
    objName.push_back("bus");
    objName.push_back("car");
    objName.push_back("cat");
    objName.push_back("chair");
    objName.push_back("cow");
    objName.push_back("diningtable");
    objName.push_back("dog");
    objName.push_back("horse");
    objName.push_back("motorbike");
    objName.push_back("person");
    objName.push_back("pottedplant");
    objName.push_back("sheep");
    objName.push_back("sofa");
    objName.push_back("train");
    objName.push_back("tvmonitor");
 

    net.setPreferableBackend(3);
    net.setPreferableTarget(1);

    VideoCapture cap;
    int deviceID = parser.get<int>("device");
    int apiID = cv::CAP_V4L2;
    // open selected camera using selected API
    cap.open(deviceID, apiID);
    //FIXME: Auto convert property failed in CSI camera(NV12 format).
    if (parser.get<int>("device") == 1)
        cap.set(CAP_PROP_CONVERT_RGB , false); 

    Mat frame;
    Mat frame_rgb24;
   
    while(waitKey(1) < 0)
    {

        if (parser.get<int>("device") == 1)
        {
            //FIXME: Workaround by cvtColor from nv12 to bgr.
            cap.read(frame_rgb24);
            cvtColor(frame_rgb24, frame, cv::COLOR_YUV2BGR_NV12);
        } else {
            cap.read(frame);
        }
        
        int h = frame.size().height;
        int w = frame.size().width;

        cv::Mat blob=dnn::blobFromImage(frame,
                                     0.007843,
                                     Size(300, 300),
                                     Scalar(127.5, 127.5, 127.5),
                                     false,false);
        net.setInput(blob);
        cv::Mat detection = net.forward("detection_out");
        /* Since the inference result is in 1*1*100*7 form, cut to 100*7 here at first. */
        cv::Mat detectionMat(detection.size[2], detection.size[3], CV_32F, detection.ptr<float>());
        //cout << detectionMat.size << endl;
        /* Threshold settings  */ 
	float confidenceThreshold = 0.2;
        for (int i = 0; i < detectionMat.rows;i++) {
            float confidence = detectionMat.at<float>(i,2);
            if (confidence> confidenceThreshold) {
                int idx = static_cast<int>(detectionMat.at<float>(i, 1));
                //cout << "idx is " << idx << endl;
                int left = static_cast<int>(detectionMat.at<float>(i, 3) * w);
                int top = static_cast<int>(detectionMat.at<float>(i, 4) * h);
                int right = static_cast<int>(detectionMat.at<float>(i, 5) * w);
                int bottom = static_cast<int>(detectionMat.at<float>(i, 6) * h);
            cv::rectangle(frame,Rect(left, top, right-left, bottom-top), Scalar(255, 0, 0), 2);
            string label = objName[idx];
            cv::putText(frame, label,cv::Point(int((left + right) / 2),int((top+bottom)/2)),
                FONT_HERSHEY_SIMPLEX,0.5, Scalar(0, 255, 0));
            }
        }

        
        vector<double> layersTimes;
        double freq = getTickFrequency() / 1000;
        double t = net.getPerfProfile(layersTimes) / freq;
        string label = format("Inference time : %.2f ms", t);
        putText(frame, label, Point(20, 40), FONT_FACE, FONT_SCALE, RED);
    
        cv::imshow("Output",frame);
        //cv::imwrite("Output.jpg",frame);
    }

    cv:: waitKey(0);

    return 0;
}
